Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities


--This Module counts the number 10 MHz clocks between BDOT inputs.
-- It then subtracts the count from the nominal count (666,666) and
-- puts the difference out as the top ten bits of a +/- 15 bit DAC

entity BDOT_Timer is
	port(
			clk_80mhz			: in std_logic; 
			nreset   	  		: in std_logic;
			BDOT_Pulse_In		: in std_logic;
		 
			Delta_T_Out_ob 	: out std_logic_vector(15 downto 0);
			Strobe_Out			: out std_logic;
			Ext_Cnt_Gate		: out std_logic;
			Clock_Out			: out std_logic
		);
end BDOT_Timer;
	
architecture main of BDOT_Timer is

signal cnt_A			: std_logic_vector(31 downto 0);
signal cnt_A_ena		: std_logic;
signal cnt_A_rst		: std_logic;

signal cnt_B			: std_logic_vector(31 downto 0);
signal cnt_B_ena		: std_logic;
signal cnt_B_rst		: std_logic;

signal bdot				: std_logic;

signal get_cnt_A		: std_logic;
signal get_cnt_B		: std_logic;

signal latched_cnt	: std_logic_vector(31 downto 0);
signal delta_cnt_2s	: std_logic_vector(31 downto 0);
signal delta_cnt_ob	: std_logic_vector(31 downto 0);

signal delta_out_ob	: std_logic_vector(31 downto 0);

signal clk_div			: std_logic_vector(2 downto 0);
signal clk				: std_logic;

component addsub32signed
	PORT
	(
		add_sub		: IN STD_LOGIC ;
		dataa			: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
		datab			: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (31 DOWNTO 0)
	);
end component;


	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(count_on_a, latch_cnt_a, reset_cnt_a, 
	 count_on_b, latch_cnt_b, reset_cnt_b  );
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 
EDGE_ON_BDOT_PULSE_IN: led
port map(sig_in => BDOT_Pulse_In, clk => clk, clrn => nreset, pls_out => bdot);


DIVIDE_CLOCK: process(clk_80mhz, nreset, clk_div)
begin
	if  (nreset = '0') then 					clk_div <= (others => '0');
	elsif rising_edge(clk_80mhz) then		clk_div <= clk_div + 1;
	end if;
end process;
-----------------------------------------------------
clk <= clk_div(2); -- 80 MHz divided by 8 = 10 MHz
Clock_Out <= clk;
-----------------------------------------------------

PROCESS_TIMER_A: process(clk, nreset, cnt_A_ena, cnt_A_rst, cnt_A)
begin
	if  (nreset = '0') then 			cnt_A <= (others => '0');
	elsif rising_edge(clk) then
		if   (cnt_A_ena = '1') then 	cnt_A <= cnt_A + 1;
		elsif(cnt_A_rst = '1') then	cnt_A <= (others => '0');
		end if;
	end if;
end process;

PROCESS_TIMER_B: process(clk, nreset, cnt_B_ena, cnt_B_rst, cnt_B)
begin
	if  (nreset = '0') then 			cnt_B <= (others => '0');
	elsif rising_edge(clk) then
		if   (cnt_B_ena = '1') then 	cnt_B <= cnt_B + 1;
		elsif(cnt_B_rst = '1') then	cnt_B <= (others => '0');
		end if;
	end if;
end process;

DAC_COUNT_LATCH: process(clk, nreset, get_cnt_A, get_cnt_B, cnt_A, cnt_B)
begin
	if  (nreset = '0') then 			latched_cnt <= (others => '0');
	elsif rising_edge(clk) then
		if   (get_cnt_A = '1') then 	latched_cnt <= cnt_A;
		elsif(get_cnt_B = '1') then	latched_cnt <= cnt_B;
		end if;
	end if;
end process;
----------------------------------------------------------
COMPUTE_DELTA: addsub32signed 
PORT MAP (
				add_sub	 	=> '0',
				dataa	 		=> X"000A2C2A",
				datab	 		=> latched_cnt,
				result	 	=> delta_cnt_2s
			);
	
	delta_cnt_ob(31 downto 0) <= (not delta_cnt_2s(31)) & delta_cnt_2s(30 downto 0);
	
LIMIT_DELTA_TO_10_BITS:
process(delta_cnt_ob)
begin
	if   (delta_cnt_ob > X"800003FF") then	delta_out_ob <= X"800003FF";
	elsif(delta_cnt_ob < X"7FFFFC01") then	delta_out_ob <= X"7FFFFC01";
	else												delta_out_ob <= delta_cnt_ob;
	end if;
end process;	

----------------------------------------------------------------------------------------------
	Delta_T_Out_ob(15 downto 0) <= (not delta_out_ob(10)) & delta_out_ob(9 downto 0) & "00000";
	
	Ext_Cnt_Gate <= cnt_A_ena;
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= count_on_a;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, bdot)
begin
	-- default outputs --
	cnt_A_ena	<= '0'; 
	cnt_A_rst	<= '0';
	
	cnt_B_ena	<= '0'; 
	cnt_B_rst	<= '0';
	
	get_cnt_A	<= '0';
	get_cnt_B	<= '0';	
	
	Strobe_Out	<= '0';
	
	case CS is
		when count_on_a   =>
			cnt_A_ena	<= '1'; 
			if (bdot = '1') then 				NS <= latch_cnt_a;	
			else										NS <= count_on_a;
			end if;

		when latch_cnt_a =>						NS <= reset_cnt_a;
			cnt_B_ena	<= '1'; 
			get_cnt_A	<= '1';
			
		when reset_cnt_a =>						NS <= count_on_b;
			cnt_B_ena	<= '1'; 
			cnt_A_rst	<= '1';
			Strobe_Out	<= '1';
			
		when count_on_b   =>
			cnt_B_ena	<= '1'; 
			if (bdot = '1') then 				NS <= latch_cnt_b;	
			else										NS <= count_on_b;
			end if;

		when latch_cnt_b =>						NS <= reset_cnt_b;
			cnt_A_ena	<= '1'; 
			get_cnt_B	<= '1';
			
		when reset_cnt_b =>						NS <= count_on_a;
			cnt_A_ena	<= '1'; 
			cnt_B_rst	<= '1';
			Strobe_Out	<= '1';
			
		when others =>								NS <= count_on_a;
	end case; 
			
end process;

	
end main;