Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities


--Version 4 breaks the EXTERNAL / MADC1 Switch logic out from the 
-- Accel_PL_Ena and Start / Stop logic.

entity Cogging_Test_Switch is
	port(clk        		: in std_logic;
		 nreset     		: in std_logic;
		 SW_1					: in std_logic; -- momentary trigger runs mode for 40ms
		 SW_2					: in std_logic; -- A High logic level sets BDOT timer mode
														-- until returned to a Low logic level
		 DIN_1				: in std_logic;
		 DIN_2				: in std_logic;
		 DIN_3				: in std_logic;
		 DIN_4				: in std_logic;
		 
		 T1					: in std_logic;
		 T2					: in std_logic;
 		 T3					: in std_logic;
		 T4					: in std_logic;
		 
		 LED_PB_SW			: in std_logic;
		 Ext_Cnt_Gate		: in std_logic;
		 clk_10MHz			: in std_logic;
		 
		 DO1					: out std_logic;
		 DO2					: out std_logic;
		 DO3					: out std_logic;
		 DO4					: out std_logic;
		 
 		 SEL_DAC				: out std_logic;
 		 SEL_EXT_A		  	: out std_logic;
 		 SEL_EXT_B			: out std_logic;
 		 SEL_MDAC2			: out std_logic;
		 
		 LED_1				: out std_logic;
		 LED_2				: out std_logic;
		 LED_3				: out std_logic;
		 LED_4				: out std_logic;
		 
		 HOLD_1				: out std_logic;
		 HOLD_2				: out std_logic;
		 
		 Beam_Cyc_Reset 	: out std_logic
		);
end Cogging_Test_Switch;
	
architecture main of Cogging_Test_Switch is

signal sel_alternate			: std_logic;
signal sel_bdot_timer_mode	: std_logic;

signal timer			: std_logic_vector(23 downto 0);
signal timer_ena		: std_logic;

signal alt_IO_mode	: std_logic;
signal cycle_reset	: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, switched, send_reset, bdot_timer_mode);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 
EDGE_ON_TEST_SELECT_TRIGGER: led
port map(sig_in => SW_1, clk => clk, clrn => nreset, pls_out => sel_alternate);

SELECT_BDOT_TIMER_MODE: process(clk, nreset, SW_2)
begin
	if  (nreset = '0') then 		sel_bdot_timer_mode <= '0';
	elsif rising_edge(clk) then
		if  (SW_2 = '1') then 		sel_bdot_timer_mode <= '1';
		else								sel_bdot_timer_mode <= '0';
		end if;
	end if;
end process;

PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 			timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '0') then 	timer <= (others => '0');
		else									timer <= timer + 1;
		end if;
	end if;
end process;


----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, timer, sel_alternate, DIN_1, DIN_2, DIN_3, DIN_4,
			T1, T2, T3, T4, 
			sel_bdot_timer_mode, Ext_Cnt_Gate, clk_10MHz)
begin
	-- default outputs --
	timer_ena 		<= '1'; -- not enabled only when in idle state
	cycle_reset		<= '0';
	
	DO1				<= T1;  -- Normal Digital Inputs to Outputs
	DO2				<= T2;
	DO3				<= T3;
	DO4				<= T4;
		
	case CS is
		when idle   =>
			timer_ena		<= '0';  
			if   (sel_alternate = '1') then 			NS <= switched;
			elsif(sel_bdot_timer_mode = '1') then	NS <= bdot_timer_mode;
			else												NS <= idle;
			end if;

		when switched =>
			if   (timer = X"30D400")    then		NS <= send_reset; -- time out at 40 ms
			else											NS <= switched;
			end if;
			
			DO1				<= DIN_1; 
			DO2				<= DIN_2;
			DO3				<= DIN_3;
			DO4				<= DIN_4;
		
		when send_reset  =>							NS <= idle;
			cycle_reset	<= '1';
			
		when bdot_timer_mode   =>
			timer_ena	<= '0';  
			DO3			<= Ext_Cnt_Gate;
			DO4			<= clk_10MHz;
			
			if   (sel_bdot_timer_mode = '1') then	NS <= bdot_timer_mode;
			else												NS <= idle;
			end if;

		when others =>									NS <= idle;
	end case; 
			
end process;

----------------------------------------------------------
SWITCH_CONTROL: process(clk, nreset, sel_alternate, cycle_reset)
begin                              -- THIS ACTS AS AN SR-FLIP FLOP
	if    (nreset = '0')          then 	alt_IO_mode <= '0';
	elsif rising_edge(clk) then 
		if   (sel_alternate = '1') then  alt_IO_mode <= '1';
		elsif(cycle_reset = '1')   then  alt_IO_mode <= '0';
		end if;
	end if;
end process; 

	Beam_Cyc_Reset	<= cycle_reset;
	
 	SEL_DAC			<= '1';		
 	SEL_EXT_A		<= '0';		
 	SEL_EXT_B		<= alt_IO_mode;		
 	SEL_MDAC2		<= not alt_IO_mode;		

	LED_1				<= SW_1;		
	LED_2				<= LED_PB_SW;		
	LED_3				<= not alt_IO_mode;		
	LED_4				<= SW_2;		
	
	HOLD_1			<= '0';
	HOLD_2			<= '0';
	
end main;