Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities


--Version 4 breaks the EXTERNAL / MADC1 Switch logic out from the 
-- Accel_PL_Ena and Start / Stop logic.

entity Enable_New_MIPL is
	port(clk        		: in std_logic;
		 nreset     		: in std_logic;
		 ARM_MIPL	 		: in std_logic;
		 Beam_Gate			: in std_logic;
		 Normal_RPOS_Hold	: in std_logic;
		 Test_RPOS_Hold		: in std_logic;
		 
 		 Module_Switch_Sel	: out std_logic;
 		 Extern_Switch_Sel  	: out std_logic;
 		 MIPL_ARMED				: out std_logic;
 		 Beam_Cyc_Reset		: out std_logic;
 		 RPOS_Hold_Out			: out std_logic

		);
end Enable_New_MIPL;
	
architecture main of Enable_New_MIPL is

signal mod_selected		: std_logic;

signal timer			: std_logic_vector(23 downto 0);
signal timer_ena		: std_logic;

signal armed			: std_logic;
signal syncd_beam_gate	: std_logic;
signal cycle_reset		: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, get_beam, cycle_active, no_beam, send_reset);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 
EDGE_ON_MODULE_SELECT_TRIGGER: led
port map(sig_in => ARM_MIPL, clk => clk, clrn => nreset, pls_out => armed);

-- We expect ARM_MIPL to show a rising edge from a TCLK timer just before
-- beam to booster.

PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 			timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '0') then 	timer <= (others => '0');
		else									timer <= timer + 1;
		end if;
	end if;
end process;

SYNC_BEAM_GATE: process(clk, nreset, Beam_Gate)
begin
	if  (nreset = '0') then 	syncd_beam_gate <= '0';
	elsif rising_edge(clk) then
										syncd_beam_gate <= Beam_Gate;
	end if;
end process;


----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, timer, armed, syncd_beam_gate, Normal_RPOS_Hold, Test_RPOS_Hold)
begin
	-- default outputs --
	timer_ena 		<= '1'; -- not enabled only when in idle state
	cycle_reset		<= '0';
	MIPL_ARMED		<= '0';
	RPOS_Hold_Out 	<= Normal_RPOS_Hold;
	
	case CS is
		when idle   =>
			timer_ena		<= '0';  
			if (armed = '1') then 				NS <= get_beam;	
			else										NS <= idle;
			end if;

		when get_beam =>
			if   (syncd_beam_gate = '1') then	NS <= cycle_active; -- we expect beam for full cycle
			elsif(timer = X"30D400")     then	NS <= send_reset; -- time out at 40 ms
			else											NS <= get_beam;
			end if;
		
		when cycle_active  =>
			MIPL_ARMED		<= '1';
			RPOS_Hold_Out 	<= Test_RPOS_Hold;

			if   (syncd_beam_gate = '0') then	NS <= no_beam; -- we expect beam for full cycle
			elsif(timer = X"30D400")     then	NS <= send_reset; -- time out at 40 ms
			else											NS <= cycle_active;
			end if;

		when no_beam  =>
			if(timer = X"30D400")   then		NS <= send_reset; -- time out at 40 ms
										-- we reset to get ready for next cycle after 40 ms
			else										NS <= no_beam;
			end if;
			
		when send_reset  =>						NS <= idle;
			cycle_reset	<= '1';
			
		when others =>								NS <= idle;
	end case; 
			
end process;

----------------------------------------------------------
SWITCH_CONTROL: process(clk, nreset, armed)
begin                              -- THIS ACTS AS AN SR-FLIP FLOP
	if    (nreset = '0')          then 	mod_selected <= '0';
	elsif rising_edge(clk) then 
		if   (armed = '1') 		  then  
													mod_selected <= '1';
		elsif(cycle_reset = '1')  then 
													mod_selected <= '0';
		end if;
	end if;
end process; 

	Module_Switch_Sel	<= mod_selected;
	Extern_Switch_Sel <= NOT mod_selected;
	Beam_Cyc_Reset		<= cycle_reset;


end main;