
Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

-- Version 3 gates of updates to the FPGA DAC except during the MIPL
-- interval.  The DAC is set to zero at End_MIPL.

-- Version 5 reduces the scope of this block to implementing the state
-- machine to combine the feedback terms.

entity MIPL_Method_2v5 is
	port(clk        		: in std_logic;
		 nreset     		: in std_logic;
		 Beam_Cyc_Reset		: in std_logic;
		 Run_MIPL			: in std_logic;
		 Input1_2s			: in std_logic_vector(31 downto 0); 
		 Input2_2s			: in std_logic_vector(31 downto 0); 
		 Update_Strobe		: in std_logic;
		 GLOBAL_OFFSET 		: in std_logic_vector(13 downto 0);
		 
		 FeedBack_Out_OB  		: out std_logic_vector(13 downto 0);
		 DAC_CLK_Out		: out std_logic
		 
		);
end MIPL_Method_2v5;
	
architecture main of MIPL_Method_2v5 is

-- Signed Value of inputs ----------------------------------
	signal BASE_2s				: std_logic_vector(13 downto 0);
	signal sum1_2s		: std_logic_vector(27 downto 0);
	signal sum2_2s		: std_logic_vector(31 downto 0);
	signal sum3_2s		: std_logic_vector(13 downto 0);

-------------------------------------------------------------
	signal  run			: std_logic;
	
	signal	dac_fb_out	: std_logic_vector(13 downto 0);
	
	signal	setup_dac	: std_logic;
	signal  strb_dac	: std_logic;
	signal  clr_dac		: std_logic;
	
----------------------------------------------------------------
	component add14signed
	PORT
	(
		dataa		: IN STD_LOGIC_VECTOR (13 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (13 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (13 DOWNTO 0)
	);
	end component;
	
	component addsub32signed
	PORT
	(
		add_sub		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (31 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (31 DOWNTO 0)
	);
	end component;
---------------------------------------------------------------

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, wait_1, calc_fb1, calc_fb2, latch_dac, strobe_dac, clear_dac, clear_dac_wr);
	signal CS, NS  : STATE_TYPE;
	

-- ## MAIN #########################################################
begin 

RESYNC_RUN: process(clk, nreset, Run_MIPL)
begin
	if   (nreset = '0')    then run <= '0';
	elsif rising_edge(clk) then run <= Run_MIPL;
	end if;
end process;

-- Convert Offset Binary Values to 2's Complement ------------------
BASE_2s		  	<= not GLOBAL_OFFSET(13)  & GLOBAL_OFFSET(12 downto 0);

			
	ADD_TERMS : addsub32signed 
	port map (
				add_sub  => '1',
				dataa	 => Input1_2s(31 downto 0),
				datab	 => Input2_2s(31 downto 0),
				result	 => sum2_2s(31 downto 0)
			 );

-- Add the Base Offset to the value we send to the DAC ----------------
	ADD_to_BASE : add14signed 
	port map (
				dataa	 => sum2_2s(31) & sum2_2s(29 downto 17), --WAS 29 DOWNTO 17 
				datab	 => BASE_2s(13 downto 0),
				result	 => sum3_2s(13 downto 0)
			 );
			 
-- State Machine Process definition ---------------------------
	sm_reg: process (clk, nreset)	
	begin
		if (nreset='0')			then CS <= clear_dac; -- <<<  Clear DAC at Reset
		elsif rising_edge(clk)	then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: 
	process (CS, run, Beam_Cyc_Reset, Update_Strobe)
	begin
		-- defaults -------
		setup_dac	<= '0';
		clr_dac		<= '0'; 
		strb_dac	<= '0';	
		
		case CS is
			when idle =>						NS <= wait_1;
--				clr_prev	<= '1';	
--				clr_acc		<= '1';
				if (run = '1') then				NS <= wait_1;
				elsif(Beam_Cyc_Reset = '1')then NS <= clear_dac;
				else							NS <= idle;
				end if;

			when wait_1 => 
				if(run = '0') then				NS <= clear_dac;
				elsif(Update_Strobe = '1') then	NS <= calc_fb1;
				elsif(Beam_Cyc_Reset = '1')then	NS <= clear_dac;
				else							NS <= wait_1;
				end if;
			
			when calc_fb1 => 					NS <= calc_fb2;
			
			when calc_fb2 => 					NS <= latch_dac;
						
			when latch_dac =>					NS <= strobe_dac;
				setup_dac 	<= '1'; -- latch new value into FeedBack_Out_OB

			when strobe_dac =>					NS <= wait_1;
				strb_dac	<= '1'; -- strobe Feedback into DAC
				
			when clear_dac => 					NS <= clear_dac_wr;
				clr_dac		<= '1';
				setup_dac 	<= '1'; -- latch cleared value into FeedBack_Out_OB
			
			when clear_dac_wr =>				NS <= idle;
				clr_dac		<= '1';
				strb_dac	<= '1'; -- strobe Feedback into DAC
			
			
			when others => 						NS <= idle;
		end case; 
				
	end process; -- sm_combinatorial

	
-- Syncronized Outputs ----------------------------

	process(clk, nreset, sum3_2s, setup_dac)
	begin
		if   (nreset = '0') then dac_fb_out <= "10000000000000";
		elsif rising_edge(clk) then
			if(setup_dac = '1') then
				if (clr_dac = '1') then dac_fb_out <= GLOBAL_OFFSET(13 downto 0);
				else
					-- convert back to 14 bit offset binary --
					dac_fb_out <= not sum3_2s(13) & sum3_2s(12 downto 0);
				end if;
			end if;
		end if;
	end process;

-- ==== NEW NEW NEW NEW NEW  ========  Added Jan 23 2012
	process(clk, nreset, strb_dac)
	begin
		if   (nreset = '0')    then DAC_CLK_Out <= '0';
		elsif rising_edge(clk) then 
			DAC_CLK_Out	<= strb_dac; -- MOVED DAC_CLK_Out 1 CLOCK LATER AFTER BUFFER
		end if;
	end process;

	FeedBack_Out_OB 	<= dac_fb_out;
	
end main;

