Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- This Version ----------------------------------------------------
--
-- In the version created on 12/28/2012 a Run_Error output was added.
-- This error is pulsed when ever the Run_MIPL processes does not
-- start within 2.5ms after the MIPL_Trigger.  The error indicates
-- that up/down slopes on the two phase detectors were not detected
-- properly.

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

-- Version 5 uses a frequency trigger produced by measuring
-- the time between zero crossings of the MI PD output triangle
-- wave.  In the full LLRF system this trigger can be replaced
-- with the similar trigger, VXIPLT, from the DDS LLRF module. 

-- Version 6 removes the logic that allowed starting MIPL on
--the upslope OR downslope of Phase Det. A.  We realized this was
--wrong, but realized that the Reference Trajectory slope could
--be compensated given the knowlege of which slope came first
--after the "Frequency Trigger" fired.  A bit to indicate the
--need for a Ref Trajectory slope adjustment is produced.

-- Version 7 eliminates the logic that waits for a rising edge
-- on the divide-by-32 PB and simply watches for the second
-- down-slope on the "A" phase detector.  A delay from this
-- event is adjustable before getting Run_MIPL and the start
-- of the Ref_Trajectory.

-- Version 8 changed sequence so that Hold_for_MIPL does not
-- trigger until the first falling edge of "Polarity" after
-- the frequency trigger fires.  Before the hold went out right
-- after the frequency trigger, but this trigger happens either
-- at an un-divided phase detector phase of -90 or +90.  We
-- believe this was causing problems for cogging.

entity MI_Phaselock_Timing_v8 is
	port(clk        		: in std_logic;
		 nreset     		: in std_logic;
		 ARM_MIPL			: in std_logic;
		 Local_Freq_Trig	: in std_logic;
		 Slope_Up_nDn_A	: in std_logic;
		 Polarity_A			: in std_logic;
		 beam_cyc_reset	: in std_logic;
		 delay_start		: in std_logic_vector(15 downto 0);
		 
		 MIPL_Armed			: out std_logic;
		 Hold_For_MIPL		: out std_logic;
		 Load_Div_Counters 	: out std_logic;
		 Run_MIPL			: out std_logic;
		 End_MIPL			: out std_logic;
		 Use_Alt_Slope		: out std_logic;
		 Run_Error			: out std_logic
		);
end MI_Phaselock_Timing_v8;
	
architecture main of MI_Phaselock_Timing_v8 is

signal run			: std_logic;
signal arm			: std_logic;
signal trig_mipl	: std_logic;
signal up_trig		: std_logic;
signal dn_trig		: std_logic;

signal hold_ena	: std_logic;
signal hold_trig	: std_logic;

--signal dn_trig_B	: std_logic;

signal timer		: std_logic_vector(23 downto 0);
signal timer_ena	: std_logic;

signal ld_div_counters	: std_logic;

signal set_alt_slope : std_logic;
signal clr_alt_slope : std_logic;


	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, armed, clear_timer_1, wait_for_dn_trig, clear_timer_2, 
		wait_for_dn_trig_2, clear_timer_3, pause1, clear_timer_4, 
		go, done, error); 
		
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 

HOLD_SIGNAL_TRIG: led
port map(sig_in => not Polarity_A, clk => clk, clrn => nreset, pls_out => hold_trig);

----------------------------------------------------------------------------
sync_ARM_Pulse: led
port map(sig_in => ARM_MIPL, clk => clk, clrn => nreset, pls_out => arm);
	
----------------------------------------------------------------------------
sync_FREQ_TRIG: led
port map(sig_in => Local_Freq_Trig, clk => clk, clrn => nreset, pls_out => trig_mipl);
	
----------------------------------------------------------------------------
sync_DN: led
port map(sig_in => not Slope_Up_nDn_A, clk => clk, 
			clrn => nreset, pls_out => dn_trig);

sync_UP: led
port map(sig_in => Slope_Up_nDn_A, clk => clk, 
			clrn => nreset, pls_out => up_trig);
--HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
HOLD_MIPL_LATCH: process(clk, nreset, hold_ena, hold_trig,beam_cyc_reset)
begin
	if  (nreset = '0')     then Hold_For_MIPL <= '0';	
	elsif rising_edge(clk) then
		if(hold_ena = '1') and (hold_trig = '1') then Hold_For_MIPL <= '1';
		elsif(beam_cyc_reset = '1') then 				 Hold_For_MIPL <= '0';
		end if;
	end if;
end process;
			
--HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
ALT_SLOPE_SELECT_LATCH: 
process(clk,nreset, set_alt_slope, clr_alt_slope )
begin
	if  (nreset = '0')     then Use_Alt_Slope <= '0';	
	elsif rising_edge(clk) then
		if   (set_alt_slope = '1') then Use_Alt_Slope <= '1';
		elsif(clr_alt_slope = '1') then Use_Alt_Slope <= '0';
		end if;
	end if;
end process;

--HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0')     then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '1') then 	timer <= timer + 1;
		else						timer <= (others => '0');
		end if;
	end if;
end process;

----------------------------------------------------------
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, arm, trig_mipl, dn_trig, up_trig, timer, delay_start)
begin
	-- default outputs --
	timer_ena 		<= '0'; 
	MIPL_Armed		<= '0';
	hold_ena			<= '0';
	ld_div_counters	<= '0';
	run				<= '0';
	End_MIPL			<= '0';
	Run_Error		<= '0';
	
	clr_alt_slope <= '0';
	set_alt_slope <= '0';
		
	case CS is
		when idle   =>
			ld_div_counters	<= '1';  -- divide-by counters are held in this case
			clr_alt_slope 	<= '1';

			if (arm = '1') 	then			NS <= armed;
			else							NS <= idle;
			end if;
	
		when armed  =>
			ld_div_counters	<= '1';
			MIPL_Armed		<= '1';

			timer_ena		<= '1';
			
			if   (timer = X"30D400") then	NS <= idle; -- time out at 40 ms
			elsif(trig_mipl = '1')   then NS <= clear_timer_1; --triggered;
			else									NS <= armed;
			end if;
 	
		when clear_timer_1 =>				NS <= wait_for_dn_trig;						
			hold_ena				<= '1'; -- enable hold and wait for trigger
			timer_ena			<= '0'; --opportunity to clear timer			
			ld_div_counters	<= '1'; 
			
		when wait_for_dn_trig => 
			hold_ena				<= '1'; -- enable hold and wait for trigger
			ld_div_counters	<= '1';
			
			timer_ena			<= '1';
			if   (timer = X"030D40") then	NS <= error; -- time out at 2.5 ms
			--if   (timer = X"0C3500") then	NS <= idle; -- time out at 10 ms
			elsif(up_trig = '1')     then 	
													set_alt_slope 	<= '1';
													NS <= wait_for_dn_trig; 
											
			elsif(dn_trig = '1')     then NS <= clear_timer_2; 
			else									NS <= wait_for_dn_trig;
			end if;
						
		when clear_timer_2 =>				NS <= wait_for_dn_trig_2; --reset_dividers;
			hold_ena				<= '1'; 
			timer_ena			<= '0';

		when wait_for_dn_trig_2 => 
			hold_ena				<= '1'; 
			timer_ena			<= '1';
			if   (timer = X"030D40") then	NS <= error; -- time out at 2.5 ms
			elsif(dn_trig = '1')   then 	NS <= clear_timer_3;
			else									NS <= wait_for_dn_trig_2;
			end if;

		when clear_timer_3 =>				NS <= pause1;
			hold_ena				<= '1'; 
			timer_ena			<= '0';

		when pause1 =>
			hold_ena				<= '1'; 
			timer_ena			<= '1';
			if   (timer(15 downto 0) = delay_start) then	
													NS <= clear_timer_4;
			else									NS <= pause1;
			end if;

		when clear_timer_4 =>				NS <= go;
			hold_ena				<= '0'; -- should be latched by now.
			timer_ena			<= '0';

		when go =>
			run			<= '1'; -- run reference trajectory and do phase lock
			timer_ena	<= '1';
			if   (timer = X"0C3500") then	NS <= done; -- wait 10 ms
			else									NS <= go;
			end if;
			
		when done  =>							NS <= idle;
			End_MIPL	<= '1';
			
		when error =>
			Run_Error 	<= '1';
			timer_ena	<= '1';
			if   (timer = X"0C3500") then	NS <= done; -- wait 10 ms
			else									NS <= error;
			end if;
			
		when others =>							NS <= idle;
	end case; 
			
end process;

-- Sync counter reset/load output
process(clk, nreset, ld_div_counters)
begin
	if   (nreset = '0')    then Load_Div_Counters <= '1';
	elsif rising_edge(clk) then Load_Div_Counters <= ld_div_counters;
	end if;
end process;

	Run_MIPL	<= run;
	
end main;