

Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

-- outputs complementary terms alpha and 1 minus alpha

-- Both Input Value and Output Value are given as 14 bit Offset Binary.

entity Ramp_Alpha_v2 is
	port(
			clk        		: in std_logic;
			nreset     		: in std_logic;
			Gate_Ramp		: in std_logic; 
			Update_Strobe	: in std_logic;
		 
			Curve_Out		: out std_logic_vector(13 downto 0);
			ramp_done 		: out std_logic
		);
end Ramp_Alpha_v2;
	
architecture main of Ramp_Alpha_v2 is

------------------------------------------------------------
-- ALTERA COMPONENTS ---------------------------------------
-- Ramp for alpha --------------------------
	component ramp_up_mem4
		PORT
		(
			address		: IN STD_LOGIC_VECTOR (7 DOWNTO 0);
			clock		: IN STD_LOGIC ;
			data		: IN STD_LOGIC_VECTOR (13 DOWNTO 0);
			wren		: IN STD_LOGIC ;
			q		: OUT STD_LOGIC_VECTOR (13 DOWNTO 0)
		);
	end component;

-----------------------------------------------------------------
	signal gate			: std_logic;
	signal alpha		: std_logic_vector(13 downto 0);
	
-- ADDR_CNT Counter/Timer declarations ---------------------------
-- The output of this counter is the address pointer
	signal nclr_adr 	: std_logic;  -- output of sm
	signal ld_adr		: std_logic;
	signal inc_adr  	: std_logic;  -- output of sm
	signal end_flag 	: std_logic;  -- output of ADDR_CNT, input to sm
	signal last_adr 	: std_logic_vector(7 downto 0);  -- input to ADDR_CNT
	signal first_adr	: std_logic_vector(7 downto 0);  -- input to ADDR_CNT
	signal addr     	: std_logic_vector(7 downto 0);  -- output of ADDR_CNT
	
	-- INTERVAL_CNT declarations ------------------------------
	signal nclr_cnt : std_logic; -- output of sm
	signal smp_flag : std_logic; -- output of INTERVAL_CNT, input to sm

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is (idle, wait_1, nextadr, hold);
	signal CS, NS  : STATE_TYPE;
	
	
	
-- ## MAIN #########################################################
begin 

--------------------------------------------------------------------------------
-- Synchronize the Gate Signal ------------------------------------

	SYNC_GATE:
	process(clk, nreset, Gate_Ramp)
	begin
		if   (nreset = '0') 	then  	gate <= '0';
		elsif rising_edge(clk) 	then	gate <= Gate_Ramp;
		end if;
	end process;
	
--------------------------------------------------------------
	ALPHA_RAMP_MEM: ramp_up_mem4
	port map(address => addr, 
			 clock => clk, 
			 data => (others => '0'),
			 wren => '0',
			 q => alpha(13 downto 0)
			);
	
	LATCH_OUTPUT:	 			 
	process(clk, nreset, Update_Strobe, alpha)
	begin
		if   (nreset = '0') 	then  	Curve_Out <= (others =>'0');
		elsif rising_edge(clk) 	then
			if   (Update_Strobe = '1') then Curve_Out <= alpha;
			end if;
		end if;
	end process;

--mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
--Memory Address Pointer -------------------------------------		
	ADDR_CNT: counter  -- address sequencer
	-- The output of this counter is the address pointer
	generic map(WID8)
	port map ( clk=>clk, clrn=>'1', ena=>inc_adr, load=>ld_adr, updn=>'1',
			   ld_data  => X"00",
			   flag_cnt => X"FE",
			   flag_out => end_flag,
			   cnt_out  => addr
			 );
	
	INTERVAL_CNT: counter  -- Defines the curve sample interval
	generic map(WID16)
	port map ( clk=>clk, clrn=>nclr_cnt, ena=>'1', load=>'0', updn=>'1',
			   ld_data  => (others => '0'),
			   flag_cnt => X"0177", -- expecting to get an 1200 us ramp of 255 points
			   flag_out => smp_flag,
			   cnt_out  => open
			 );

---------------------------------------------------------------		
-- State Machine Process definition ---------------------------
	sm_reg: process (clk, nreset)	
	begin
		if(nreset='0')              then CS <= idle;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: process (CS, smp_flag, end_flag, gate)
	begin
		--nclr_adr <= '1'; -- Changed Aug 10, loading instead of clearing -- counter is reset 
		nclr_cnt 	<= '1'; -- counter is not reset by default
		inc_adr  	<= '0'; -- not incrementing the address pointer
		ld_adr	 	<= '0';
		ramp_done 	<= '0';
		
		case CS is
			when idle   =>
				ld_adr	 <= '1';
				nclr_cnt <= '0'; -- clear the interval count
				if   (gate ='1') then 		NS <= wait_1  ;
				else                      	NS <= idle;
				end if; 
				
			when wait_1 => -- wait for next sample time
				if   (smp_flag ='1') then 	NS <= nextadr;
				elsif(gate ='0')then 		NS <= idle;
				else                      	NS <= wait_1;
				end if;
				
			when nextadr => 
				if   (end_flag ='1') then 	NS <= hold;
				elsif(gate ='0') then 		NS <= idle;
				else                      	NS <= wait_1;
				end if;

				nclr_cnt <= '0'; -- clear the interval count
				inc_adr  <= '1';  -- enable the addr count
				
			when hold => -- Hold last address until end of cycle
				ramp_done <= '1';
				if   (gate ='0') then 		NS <= idle;
				else                      	NS <= hold;
				end if;
				
		end case; 
				
	end process; -- sm_combinatorial
end main;

