--------------------------------------------------------------------
--  This routine is used to manage the function of a 
--  momentary pushbutton.  I will initiate a "save all"
--  to ROM memory task when push briefly and it will
--  generate a syncronized board reset if pushed for
--  a longer duration.

Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;
USE work.pkg_081006.all;  -- my definitions and entities

-------------------------------------------------------------------- 

entity UI_Utility is
	port(clk   			: in std_logic;
		 PB_SW 			: in std_logic;
		 USB_Save_All	: in std_logic;
		 
		 rst_out 		: out std_logic;
		 save   		: out std_logic
		);
end UI_Utility;
	
architecture main of UI_Utility is
	
	signal press : std_logic;
	-- Counter/Timer declarations --------------------------------------
	signal nclr_cnt : std_logic;
	signal got_flag: std_logic;
	signal time_mark : std_logic_vector(31 downto 0);

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(init, idle, t1, t2, do_save, snd_rst);
	signal CS, NS  : STATE_TYPE;
	
	signal usb_save : std_logic;
	signal rst_now 	: std_logic;
	signal save_now	: std_logic; 

	-- Registers to time reset pulse out
	signal qrst1,qrst2,qrst3 : std_logic;
	signal nreset_sm : std_logic;

begin -- main --------------------------------------------------------
		
	SUMCNT: counter  -- Timer
	generic map(WID32)
	port map ( clk=>clk, clrn=>nclr_cnt, ena=>'1', load=>'0', updn=>'1',
			   ld_data  => (others => '0'),
			   flag_cnt => time_mark,
			   flag_out => got_flag,
			   cnt_out  => open
			 );

	INPSYNC: sync  -- Syncronizes the push button input
	port map(sig_in => PB_SW, clk => clk, clrn => nreset_sm, sig_out => press );
	
	SYNC_USB_SAVE: led
	port map(sig_in => USB_Save_All, clk => clk, clrn => nreset_sm, pls_out => usb_save);
	
	
-- State Machine definition ---------------------------
	sm_reg: process (clk, nreset_sm)	
	begin
		if(nreset_sm='0')           then CS <= init;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: process (CS, press, got_flag)
	begin
		case CS is
			when init   =>
				NS <= idle  ;
				-- outputs --
				nclr_cnt <= '0';
				time_mark <= (others => 'X');
				rst_now <= '0';
				save_now <='0';
				--------------------
			when idle   =>
				if(press='0')then NS <= t1  ;
				else           	  NS <= idle;
				end if;
				-- outputs --
				nclr_cnt <= '0';
				time_mark <= X"00013880";
				rst_now <= '0';
				save_now <='0';
				--------------------
			when t1 => -- This state looks for "stable" button press
				if(press='1')then 			NS <= idle;
				elsif(got_flag='1') then 	NS <= t2;
				else                   		NS <= t1;
				end if;
				-- outputs --
				nclr_cnt <= '1';
				time_mark <= X"00013880";
				rst_now <= '0';
				save_now <='0';
				--------------------
			when t2 => 
				if(press='1')then 			NS <= do_save;
				elsif(got_flag='1') then 	NS <= snd_rst;
				else                   		NS <= t2;
				end if;
				-- outputs --
				nclr_cnt <= '1';
				time_mark <= X"07270E00";
				rst_now <= '0';
				save_now <='0';
				--------------------
			when do_save => -- A momentary press will do a memory save
				NS <= idle; 
				-- outputs --
				nclr_cnt <= '0';
				time_mark <= (others => 'X');
				rst_now <= '0';
				save_now <='1';
				----------------
			when snd_rst => -- A two second press will reset the module
				NS <= idle;
				-- outputs --
				nclr_cnt <= '0';
				time_mark <= (others => 'X');
				rst_now <= '1';
				save_now <='0';
				----------------
		end case; 
				
	end process; -- sm_combinatorial
	
	process(clk)
	begin
		if rising_edge(clk) then
			qrst1 <= rst_now;
			qrst2 <= qrst1;
			qrst3 <= qrst2;
			--------------------
			save <= save_now OR usb_save;
			--------------------
			nreset_sm 	<= not(qrst1 or qrst2 or qrst3);
			rst_out 	<= (qrst1 or qrst2 or qrst3);
			--------------------
		end if;
	end process;


end main;

